@@ -1,6 +1,6 @@
 /***************************************************************************
                              SRC/mixmod/Kernel/Parameter/GaussianEDDAParameter.cpp  description
-    copyright            : (C) MIXMOD Team - 2001-2014
+    copyright            : (C) MIXMOD Team - 2001-2016
     email                : contact@mixmod.org
  ***************************************************************************/
 
@@ -295,6 +295,8 @@ void GaussianEDDAParameter::initUSER(Parameter * iParam) {
 	// because of the implementation in class XEMStrategyType
 	// in gaussian cases the init parameters are allways General
 	GaussianGeneralParameter * param = (GaussianGeneralParameter *) iParam->getGaussianParameter();
+//GaussianParameter* paramTemp = iParam->getGaussianParameter();
+	//GaussianGeneralParameter * param = static_cast<GaussianGeneralParameter*>(iParam->getGaussianParameter(););
 	double ** iTabMean = param->getTabMean();
 	double * iTabProportion = param->getTabProportion();
 	Matrix ** iTabWk = param->getTabWk();
@@ -320,11 +322,35 @@ void GaussianEDDAParameter::input(std::ifstream & fi) {
 	for (k = 0; k < _nbCluster; k++) {
 		muk = _tabMean[k];
 		// Proportions //
-		fi >> _tabProportion[k];
+		_tabProportion[k] = getDoubleFromStream(fi);
+
 		// Center (mean) //
-		for (j = 0; j < _pbDimension; j++) {
-			fi >> muk[j];
-		}
+		for (j = 0; j < _pbDimension; j++)
+			muk[j] = getDoubleFromStream(fi);
+		// Variance matrix //
+		_tabSigma[k]->input(fi); // virtual method
+	} // end for k
+}
+
+//For Heterogeneous model.
+void GaussianEDDAParameter::input(std::ifstream & fi, int64_t nbVariables_binary, std::vector< int64_t > nbFactor) {
+
+	int64_t j, k;
+	double * muk;
+  double garbage;
+  int64_t sumNbFactor = 0;
+  for (int64_t l = 0; l < nbFactor.size(); l++) sumNbFactor += nbFactor[l]; 
+  for (int t = 0; t < _nbCluster * (1 + nbVariables_binary + sumNbFactor); t++) { 
+    garbage = getDoubleFromStream(fi);
+  }
+	for (k = 0; k < _nbCluster; k++) {
+		muk = _tabMean[k];
+		// Proportions //
+		_tabProportion[k] = getDoubleFromStream(fi);
+
+		// Center (mean) //
+		for (j = 0; j < _pbDimension; j++)
+			muk[j] = getDoubleFromStream(fi);
 		// Variance matrix //
 		_tabSigma[k]->input(fi); // virtual method
 	} // end for k
@@ -512,7 +538,8 @@ void GaussianEDDAParameter::edit(std::ofstream & oFile, bool text) {
 	}
 	else {
 		for (k = 0; k < _nbCluster; k++) {
-			oFile << _tabProportion[k] << endl;
+			putDoubleInStream(oFile, _tabProportion[k]);
+
 			editTab(_tabMean + k, 1, _pbDimension, oFile, " ", "");
 			_tabSigma[k]->edit(oFile, "");
 			oFile << endl;
